import os
import re
import json
import kfp
import time
import subprocess
import requests
from kubernetes import client, config
from datetime import datetime
from urllib.parse import quote_plus


class RunPipe:
    """This is end-to-end kubeflow process.

    Generate kubeflow pipeline configuration.
    Run kubeflow experiment.
    Monitoring kubeflow experiment job.
    Send notification for job status.

    Attributes:
        params (dict): parameters that will be used on pipeline.

    """

    def __init__(self, params=None):
        """Construct for storing class attributes.

        Args:
            params_path (str): location of component params.

        """
        self.params = params
        config.load_incluster_config()

    def get_pipeline_list(self):
        """Get all pipelines that created on kubeflow platform.

        Returns:
            dict: list of pipelines info.

        """
        kfp_client = kfp.Client()
        pipeline_data = {}
        page_token = None

        while True:
            pipeline_list = kfp_client.list_pipelines(
                page_token=page_token, page_size=5
            )

            for pipe in pipeline_list.pipelines:
                pipeline_data[pipe.name] = pipe.id

            page_token = pipeline_list.next_page_token
            if page_token is None:
                break

        return pipeline_data

    def get_pipeline_version_list(self, pipeline_id):
        """Get all pipeline versions that created on kubeflow platform.

        Args:
            pipeline_id (str): pipeline ID that used for checking versions.

        Returns:
            dict: list of pipeline versions info.

        """
        kfp_client = kfp.Client()
        pipe_versions = {}
        page_token = None

        while True:
            version_list = kfp_client.pipelines.list_pipeline_versions(
                resource_key_id=pipeline_id, page_token=page_token, page_size=5
            )
            version_list = version_list.to_dict()

            for version in version_list["versions"]:
                pipe_versions[version["name"]] = version["id"]

            page_token = version_list["next_page_token"]
            if page_token is None:
                break

        return pipe_versions

    def check_any_pipeline(self, local_path, pipeline_data):
        """Checking if any pipeline.

        Prevent create duplicate pipeline and pipeline version.
        If they do not exist, than create new pipeline

        Args:
            local_path (str): location of pipeline config file.
            pipeline_data (dict): list of pipelines info.

        Returns:
            :rtype: (str, str): pipeline or version id and category id.

        """
        pipe_name = self.params["PIPE_NAME"]
        pipe_ver_name = "{pipe_name}-{version}".format(
            pipe_name=self.params["PIPE_NAME"],
            version=os.getenv("CI_COMMIT_SHA")
        )

        try:
            pipeline_id = pipeline_data[pipe_name]
            pipe_versions = self.get_pipeline_version_list(pipeline_id)
            kfp_client = kfp.Client()

            try:
                return (pipe_versions[pipe_ver_name], "ver_id")
            except KeyError:
                res_ver = kfp_client.pipeline_uploads.upload_pipeline_version(
                    local_path, pipelineid=pipeline_id, name=pipe_ver_name
                )

                return (res_ver.id, "ver_id")
        except KeyError:
            print("there's no pipeline selected, try to create new...")
            kfp_client = kfp.Client()
            res_pipe = kfp_client.upload_pipeline(local_path, pipe_name)

            return (res_pipe.id, "pipe_id")

    def check_any_experiment(self):
        """Checking if any experiment.

        Prevent create duplicate experiment.
        If the experiment doesn't exist, than create new one.

        Returns:
            str: experiment id.

        """
        exp_name = self.params["EXP_NAME"]
        exp_desc = self.params["EXP_DESC"]

        try:
            kfp_client = kfp.Client()
            res_exp = kfp_client.get_experiment(experiment_name=exp_name)
            exp_id = res_exp.id
        except Exception:
            kfp_client = kfp.Client()
            res_exp = kfp_client.create_experiment(
                exp_name, description=exp_desc
            )
            exp_id = res_exp.id

        return exp_id

    def run_logging(self, pod_names, prev_time, cur_time):
        """Print logging from stackdriver.

        Args:
            pod_names (list): List of pod names.
            prev_time (str): Start time of logging.
            cur_time (str): End time of logging.

        """
        pod_name_filter = ""
        for i, pod_name in enumerate(pod_names):
            if i == 0:
                pod_name_filter += pod_name
            else:
                pod_name_filter += " OR {pod_name}".format(
                    pod_name=pod_name
                )

        cmd_str = """gcloud logging read \
            'timestamp>=\"{prev_time}\" AND timestamp<=\"{cur_time}\"
            AND resource.type=k8s_container
            AND resource.labels.cluster_name=kubeflow-research
            AND resource.labels.pod_name=({pod_name})' \
            --order=asc --format=json | python3 -c \
            'import sys, json; [print(data[\"textPayload\"]) \
            for data in json.load(sys.stdin)]'""".format(
            prev_time=prev_time,
            cur_time=cur_time,
            pod_name=pod_name_filter,
        )

        os.system(cmd_str)

    def send_notification(self, nodes, comps, status, job_name, log_url):
        """Get notification into workchat channels.

        Get notification of logging stackdriver url and job status.

        Args:
            nodes (dict): List of component job description.
            comps (list): List of pod names.
            status (str): Component job status.
            job_name (str): Experiment job name.
            log_url (str): Stackdriver logging url.

        Returns:
            type: Description of returned object.

        """
        base_url = "https://api.warungpintar.co/warbot/v1/send?recipient="
        url = (
            "{base_url}{workchat_id}&type=\
            {workchat_type}&gitlab_token={token}".format(
                base_url=base_url,
                workchat_id=self.params["WORKCHAT_ID"],
                workchat_type=self.params["WORKCHAT_TYPE"],
                token=self.params["GITLAB_PERS_TOKEN"],
            )
        )
        headers = {"content-type": "application/json"}

        comp_status = ""

        for i, comp in enumerate(comps):
            sel_node = nodes[comp]
            sel_comp_stat = """<COMPONENT-{n_comp}>
            COMP_NAME: {dis_name}
            STATUS: {phase}
            FINISH_DATE: {finish_time}\n\n""".format(
                n_comp=str(i + 1),
                dis_name=sel_node["displayName"],
                phase=sel_node["phase"],
                finish_time=sel_node["finishedAt"],
            )
            comp_status += sel_comp_stat

        msg = """<<<{job_name}>>>\n
        JOB_STATUS: {status}
        LOGGING URL: {log_url}\n
        {comp_status}""".format(
            job_name=job_name, status=status,
            log_url=log_url, comp_status=comp_status
        )
        data = {"message": msg}
        json_data = json.dumps(data)

        requests.post(url, headers=headers, data=json_data)

    def get_namespace_and_pods(self, pipe_run_id):
        """Get namespace and pod names that run on kubeflow.

        Args:
            pipe_run_id (str): Pipeline job id.

        Returns:
            list: List of pod names.

        """
        while True:
            namespaces = []

            v1 = client.CoreV1Api()
            ret = v1.list_pod_for_all_namespaces(watch=False)
            for i in ret.items:
                namespaces.append((i.metadata.namespace, i.metadata.name))

            kfp_client = kfp.Client()
            response = kfp_client.get_run(run_id=pipe_run_id)
            manifest = json.loads(response.pipeline_runtime.workflow_manifest)
            pattern = manifest["metadata"]["name"]

            pods = [ns[1] for ns in namespaces if re.match(pattern, ns[1])]

            if len(pods) > 0:
                break
            else:
                print("pods not found, trying to search pods...")
                time.sleep(1)

        return pods

    def check_job(self, pipe_run_id):
        """Check pipeline job whether is success or fail.

        Args:
            pipe_run_id (str): Pipeline job id.

        Returns:
            dict: Description of pipeline job.

        """
        job_details = {"nodes": None, "job_status": None, "job_name": None}

        try:
            kfp_client = kfp.Client()
            response = kfp_client.wait_for_run_completion(
                run_id=pipe_run_id, timeout=5
            )

            manifest = json.loads(response.pipeline_runtime.workflow_manifest)

            job_details["nodes"] = manifest["status"]["nodes"]
            job_details["job_status"] = manifest["status"]["phase"]
            job_details["job_name"] = response.run.name

            return job_details
        except Exception:
            return job_details

    def wait_job_until_finish(self, pipe_run_id, comps, log_url):
        """Checking job when pipeline job success or failed.

        Args:
            pipe_run_id (str): Pipeline job id.
            comps (type): List of pod names.
            log_url (type): Stackdriver logging url.

        """
        prev_time = datetime.now().isoformat(timespec="seconds") + "Z"
        job_details = self.check_job(pipe_run_id)
        is_finished = False

        while True:
            if job_details["job_status"] == "Succeeded":
                is_finished = True

            if (
                job_details["job_status"] == "Succeeded"
                or job_details["job_status"] == "Failed"
            ):
                self.send_notification(
                    job_details["nodes"],
                    comps,
                    job_details["job_status"],
                    job_details["job_name"],
                    log_url,
                )
                break

            time.sleep(1)
            cur_time = datetime.now().isoformat(timespec="seconds") + "Z"
            self.run_logging(comps, prev_time, cur_time)

            job_details = self.check_job(pipe_run_id)
            prev_time = cur_time

        if not is_finished:
            raise Exception("Job is failed, please check your logs...")

    def generate_log_url(self, pod_names):
        """Modify stackdriver url with selected pod names.

        Args:
            pod_names (list): List of pod names.

        Returns:
            str: final stackdriver url after modified.

        """
        pod_name_filter = ""
        for i, pod_name in enumerate(pod_names):
            if i == 0:
                pod_name_filter += '"{pod_name}"'.format(
                    pod_name=pod_name
                )
            else:
                pod_name_filter += ' OR "{pod_name}"'.format(
                    pod_name=pod_name
                )

        base_url = "https://console.cloud.google.com/logs/viewer?"
        filter_url = "project=warung-support&interval=NO_LIMIT&advancedFilter="
        query_str = 'resource.type="k8s_container" \n\
            AND resource.labels.cluster_name="kubeflow-research" \n\
            AND resource.labels.pod_name={pod_name}'.format(
            pod_name=pod_name_filter
        )

        query = quote_plus(query_str, safe="()")
        final_url = base_url + filter_url + query

        return final_url

    def run_pipe(self):
        """All process when running job on kubeflow."""
        pipeline_data = self.get_pipeline_list()

        # check if any existing pipeline
        pipe_id, id_type = self.check_any_pipeline(
            self.params["PIPE_PATH"], pipeline_data
        )

        # check if any existing experiment
        exp_id = self.check_any_experiment()
        kfp_client = kfp.Client()
        job_name = "{exp_name} experiment {commit_sha}".format(
            exp_name=self.params["EXP_NAME"],
            commit_sha=os.getenv("CI_COMMIT_SHA")
        )

        if id_type == "pipe_id":
            pipe_run = kfp_client.run_pipeline(
                exp_id, job_name, pipeline_id=pipe_id
            )
        elif id_type == "ver_id":
            pipe_run = kfp_client.run_pipeline(
                exp_id, job_name, version_id=pipe_id
            )

        pod_names = self.get_namespace_and_pods(pipe_run.id)
        # generate stackdriver logging url
        log_url = self.generate_log_url(pod_names)

        self.wait_job_until_finish(pipe_run.id, pod_names, log_url)

    def check_node_pool_label(self):
        """Checking node pool whether exist or not.

        Returns:
            bool: Node pool status.

        """
        all_labels = []
        sel_labels = []

        cmd_str = "gcloud container node-pools list \
        --cluster=kubeflow-research --zone=asia-east1-a \
        --format=json"

        process = subprocess.Popen(
            [cmd_str], shell=True,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        out, err = process.communicate()
        out = out.decode("UTF8")
        nodes = json.loads(out)

        for node in nodes:
            labels = node["config"]["labels"]
            all_labels.append(labels)

        for c_comp in self.params["COMPONENTS"]:
            node_key = c_comp["NODE_KEY"]
            node_val = c_comp["NODE_VAL"]

            sel_label = list(
                (
                    item
                    for item in all_labels
                    if node_key in item.keys() and node_val in item.values()
                )
            )
            sel_labels += sel_label

        if len(sel_labels) == len(self.params["COMPONENTS"]):
            print(
                "node label {node_key}: {node_val} is existed".format(
                    node_key=node_key, node_val=node_val
                )
            )
            return True
        else:
            print(
                "node label {node_key}: {node_val} is not existed".format(
                    node_key=node_key, node_val=node_val
                )
            )
            return False

    def check_image_tag(self):
        """Check image tag whether exist or not.

        Returns:
            bool: image tag status.

        """
        status = True

        for c_comp in self.params["COMPONENTS"]:
            cmd_str = 'gcloud container images list \
            --repository={img_registry} \
            --filter="name:{img_name}"'.format(
                img_registry=self.params["IMAGE_REGISTRY"],
                img_name=c_comp["IMG_NAME"]
            )

            process = subprocess.Popen(
                [cmd_str], shell=True,
                stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
            out, err = process.communicate()
            out = out.decode("UTF8").split("\n")

            if len(out) == 1:
                status = False
                print(
                    "image tag {img_name} is not existed".format(
                        img_name=c_comp["IMG_NAME"]
                    )
                )
                break
            else:
                print(
                    "image tag {img_name} is existed".format(
                        img_name=c_comp["IMG_NAME"]
                    )
                )
                continue

        return status

    def main(self):
        """All process run on kubeflow."""
        if not self.check_image_tag():
            raise Exception("Image tag not found...")

        if not self.check_node_pool_label():
            raise Exception("node pool label not found...")

        self.run_pipe()



import os
import kfp.dsl as dsl
import ruamel.yaml as yaml
from kfp.compiler import Compiler
from kubextract.utils.watcher import RunPipe

with open(os.getenv("PARAMS_PATH"), "r") as stream:
    PARAMS = yaml.safe_load(stream)


def create_comp(c_params):
    """Create configuration of component on kubeflow pipeline.

    Args:
        c_params (dict): Attributes of component.

    Returns:
        obj: Container object.

    """
    try:
        pipe_out = c_params["PIPE_OUTPUT"]
    except KeyError:
        pipe_out = {}

    return dsl.ContainerOp(
        name=c_params["COMP_NAME"],
        image="{img_reg}/{img_name}:{tag}".format(
            img_reg=PARAMS["IMAGE_REGISTRY"],
            img_name=c_params["IMG_NAME"],
            tag=os.getenv("CI_COMMIT_REF_NAME"),
        ),
        file_outputs=pipe_out,
    )


@dsl.pipeline(name=PARAMS["PIPE_NAME"], description=PARAMS["PIPE_DESC"])
def cluster_pipeline():
    """Create component flow on kubeflow pipeline."""
    for c_params in PARAMS["COMPONENTS"]:
        if c_params["USE_GPU"]:
            prep_op = create_comp(c_params).set_gpu_limit(1)
        else:
            prep_op = create_comp(c_params)

        prep_op.add_node_selector_constraint(
            c_params["NODE_KEY"], c_params["NODE_VAL"]
        )
        prep_op.set_image_pull_policy("Always")


def main():
    """Entrypoint of pipeline."""
    Compiler().compile(cluster_pipeline, PARAMS["PIPE_PATH"])

    gen_pipe = RunPipe(params=PARAMS)
    gen_pipe.main()


if __name__ == "__main__":
    main()



from prompt_toolkit.validation import Validator, ValidationError
from PyInquirer import prompt
import click
import ruamel.yaml as yaml
import os
import re


class StringValidator(Validator):
    """Checking input for avoiding error."""

    def validate(self, document):
        """Checking input for empty and TypeVariable.

        Args:
            document (obj): input object of CLI.

        """
        if document.text == "":
            raise ValidationError(message="Please do not input empty string")

        try:
            str(document.text)
        except Exception:
            inp_type = str(str)
            raise ValidationError(
                message="Please input {inp_type} type".format(
                    inp_type=inp_type
                )
            )


QUES = [
    {
        "type": "input",
        "name": "params_path",
        "message": "Where's your params yaml path:",
        "validate": StringValidator,
    }
]


class GenerateFrame:
    """Automatically extract kubeflow component framework.

    This extraction include all kubeflow, docker and gitlab-ci configuration.

    Args:
        params_path (str): location of component params.

    Attributes:
        params (dict): List of component params.
        build_template (str): template of build image config file.
        kubepipe_template (str): template of kubepipe deployment config file.
        kfserve_template (str): template of kfserving deployment config file.
        main_ci_template (str): template of main gitlab-ci config file.
        cloudbuild_template (str): template of cloudbuild config file.
        params_path

    """

    def __init__(self, params_path=None):
        """Construct for storing class attributes.

        Args:
            params_path (str): location of component params.

        """
        self.params = self.load_params(params_path)
        self.params_path = params_path

        dir_name = os.path.dirname(os.path.abspath(__file__))
        dir_name = dir_name.replace("utils", "templates")

        self.build_template = os.path.join(
            dir_name, "_build-gitlab-ci.yml"
        )
        self.kubepipe_template = os.path.join(
            dir_name, "_kubepipe-gitlab-ci.yml"
        )
        self.kfserve_template = os.path.join(
            dir_name, "_kfserving-gitlab-ci.yml"
        )
        self.main_ci_template = os.path.join(
            dir_name, "_gitlab-ci.yml"
        )
        self.cloudbuild_template = os.path.join(
            dir_name, "_cloudbuild.yaml"
        )

    def load_params(self, path):
        """Read params file for component params.

        Args:
            path (str): location of params file.

        Returns:
            dict: component params.

        """
        with open(path, "r") as stream:
            params = yaml.safe_load(stream)

        return params

    def add_build_ci_env(self, build_str, comp):
        """Add environment on subtitutions gcloud build flag.

        In order to add additional args when building image.

        Args:
            build_str (str): Template of gcloud build on gitlab-ci.
            comp (dict): Attributes of component.

        Returns:
            str: Template of gcloud build have been modified.

        """
        space = " " * 6

        try:
            for env_var in comp["CI_ENV"]:
                build_str += ",\\\n{space}_{key}=${value}".format(
                    key=env_var, value=env_var, space=space
                )

            build_str += "\n"
        except KeyError:
            build_str += "\n"

        return build_str

    def generate_ci_build_conf(self):
        """Modify configuration of build stage on gitlab-ci.

        Variables on the template will be changed into params values.
        Also, add new string as variables on subtitutions.

        Returns:
            str: Final configuration of build stage

        """
        final_str = ""

        for i, c_comp in enumerate(self.params["COMPONENTS"]):
            with open(self.build_template, "r") as rfile:
                build_str = rfile.read()
                build_str_len = len(build_str)

                if build_str[build_str_len - 1] == "\n":
                    build_str = build_str[: build_str_len - 1]

                build_str = build_str.format(
                    stage_name=c_comp["STAGE_NAME"],
                    local_src=self.params["LOCAL_SRC"],
                    comp_path=c_comp["COMP_PATH"],
                    image_registry=self.params["IMAGE_REGISTRY"],
                    image_tag=c_comp["IMG_NAME"],
                )

                build_str = self.add_build_ci_env(build_str, c_comp)

                if i == 0:
                    final_str += build_str
                else:
                    build_str = "\n" + build_str
                    final_str += build_str

        return final_str

    def generate_kubepipe_conf(self, path):
        """Configuration on deploy-pipe and deploy stage.

        Deploy-pipe stage use kubepipe template for training.
        Deploy stage use kfserving template for inference.
        Both templates contain variables will be changed into params values.

        Args:
            path (str): Path of kubepipe or kfserving template.

        Returns:
            str: Final configuration of kubepipe or kfserving.

        """
        with open(path, "r") as rfile:
            kpipe_str = rfile.read()
            kpipe_str_len = len(kpipe_str)
            kpipe_str = kpipe_str[: kpipe_str_len - 1]

            kpipe_str = kpipe_str.format(
                stage_name=self.params["PIPE_NAME"],
                params_path=self.params_path
            )

        return kpipe_str

    def generate_stage_conf(self):
        """Load main of gitlab-ci template.

        It contains of build_content and kubepipe_content variables.
        Both contents contain of build, deploy-pipe, and deploy stage config.

        Returns:
            str: content of gitlab-ci main template.

        """
        with open(self.main_ci_template, "r") as rfile:
            stage_str = rfile.read()

        return stage_str

    def generate_ci_conf(self, filepath):
        """All processes of modifying gitlab-ci config.

        First, load all template of part of gitlab-ci config.
        Second, modify all variables with params values.
        Third, add string as additional variables on gcloud build if any
        Last, combine all part of gitlab-ci and create gitlab-ci.yaml file

        Args:
            filepath (str): Filename of gitlab-ci config.

        """
        stage_str = self.generate_stage_conf()
        build_str = self.generate_ci_build_conf()

        if self.params["PIPE_TYPE"] == "kfpipe":
            kpipe_str = self.generate_kubepipe_conf(self.kubepipe_template)
        elif self.params["PIPE_TYPE"] == "kfserving":
            kpipe_str = self.generate_kubepipe_conf(self.kfserve_template)

        final_str = stage_str.format(
            build_content=build_str, kubepipe_content=kpipe_str
        )

        if not os.path.exists(self.params["LOCAL_SRC"]):
            os.makedirs(self.params["LOCAL_SRC"])

        target_path = "{local_src}/{filepath}".format(
            local_src=self.params["LOCAL_SRC"], filepath=filepath
        )
        with open(target_path, "w") as wfile:
            wfile.write(final_str)

        click.echo(
            "\ncreating {filepath} to {target_path}...".format(
                filepath=filepath, target_path=target_path
            )
        )

    def add_cloudbuild_ci_env(self, cloud_str, c_comp):
        """Modify cloudbuild template on substitutions and build-args.

        Before use var on docker, you have add var on cloudbuild.
        New variables additional command is added on both section.

        Args:
            cloud_str (str): Content of cloudbuild template.
            c_comp (dict): Attributes of component.

        Returns:
            str: Final modification of cloudbuild config.

        """
        split_str = cloud_str.split("\n")
        subs_keys = []
        new_args = []

        try:
            for env_var in c_comp["CI_ENV"]:
                subs_key = "{space}_{key}: {value}".format(
                    key=env_var, value=env_var, space=" " * 4
                )
                new_arg = "{space}'--build-arg', \
                '{key}=${{_{value}}}',".format(
                    key=env_var, value=env_var, space=" " * 11
                )

                subs_keys.append(subs_key)
                new_args.append(new_arg)

            split_str[9:9] = subs_keys
            split_str[3:3] = new_args
            cloud_str = "\n".join(split_str)

            return cloud_str
        except KeyError:
            return cloud_str

    def generate_cloudbuild(self, filepath):
        """All processes of modifying cloudbuild config.

        First, load cloudbuild template.
        Second, add new var additional command if any.
        Last, create all cloudbuild.yaml file on each component.

        Args:
            filepath (str): Filename of cloudbuild config.

        """
        for i, c_comp in enumerate(self.params["COMPONENTS"]):
            with open(self.cloudbuild_template, "r") as rfile:
                cloud_str = rfile.read()
                cloud_str = self.add_cloudbuild_ci_env(cloud_str, c_comp)

                target_path = "{local_src}/{comp_path}".format(
                    local_src=self.params["LOCAL_SRC"],
                    comp_path=c_comp["COMP_PATH"]
                )

                if not os.path.exists(target_path):
                    os.makedirs(target_path)

                target_file = "{target_path}/{filepath}".format(
                    target_path=target_path, filepath=filepath
                )
                with open(target_file, "w") as wfile:
                    wfile.write(cloud_str)

                click.echo(
                    "\ncreating {filepath} to {target_path}...".format(
                        filepath=filepath, target_path=target_path
                    )
                )

    def read_dockerfile(self, path):
        """Load existing dockerfile on component.

        Args:
            path (str): Location of component dockerfile.

        Returns:
            str: Content of component dockerfile.

        """
        with open(path, "r") as rfile:
            content = rfile.read()
            content = content.split("\n")

        return content

    def write_dockerfile(self, path, content):
        """Create existing dockerfile after modifying content.

        Args:
            path (str): Target location of modified dockerfile.
            content (str): Final content of dockerfile after modified.

        """
        with open(path, "w") as wfile:
            wfile.write(content)

    def add_dockerfile_env(self, dock_cont, c_comp):
        """Modify dockerfile configuration.

        Setting workdir, add source code, and env variables.

        Args:
            dock_cont (str): Description of parameter `dock_cont`.
            c_comp (dict): Attributes of component.

        Returns:
            str: Dockerfile content after modified.

        """
        envs = ["ENV PARAMS_PATH=params.yml"]
        workdirs = [
            "WORKDIR {workdir}".format(workdir=c_comp["DOCKER_WORKDIR"]),
            "ADD {comp_path} {workdir}".format(
                workdir=c_comp["DOCKER_WORKDIR"], comp_path=c_comp["COMP_PATH"]
            ),
            "ADD params.yml {workdir}/params.yml".format(
                workdir=c_comp["DOCKER_WORKDIR"]
            ),
        ]

        dock_cont = self.check_workdir_ci(dock_cont, c_comp["DOCKER_WORKDIR"])

        try:
            for env_var in c_comp["CI_ENV"]:
                dock_arg = "ARG {arg}".format(arg=env_var)
                dock_env = "ENV {key}=${{{value}}}".format(
                    key=env_var, value=env_var
                )

                envs += [dock_arg, dock_env]
        except KeyError:
            print("There are no CI environment variables...")

        for env_var in envs:
            dock_cont = self.check_dockerfile_ci_env(dock_cont, env_var)

        all_new_cont = envs + workdirs
        dock_cont[1:1] += all_new_cont

        dock_cont = self.normalize_new_line(dock_cont)
        dock_cont = "\n".join(dock_cont)
        dock_cont += "\n"

        return dock_cont

    def normalize_new_line(self, dock_cont):
        """Automatically fix space on dockerfile content.

        Args:
            dock_cont (str): dockerfile content after modified.

        Returns:
            str: dockerfile content after fix space.

        """
        new_cont = []

        for i, cont in enumerate(dock_cont):
            if i == 0:
                new_cont.append(cont)
                continue

            if cont[0] == " ":
                new_cont.append(cont)
            else:
                new_cont += ["", cont]

        return new_cont

    def check_workdir_ci(self, dock_cont, re_filter):
        """Check if any workdir config on dockerfile.

        Fix workdir component whatever exist or not.

        Args:
            dock_cont (str): content of dockerfile template.
            re_filter (str): string for regex filter.

        Returns:
            list: list of new content for dockerfile.

        """
        new_cont = []

        for cont in dock_cont:
            if cont == "":
                continue
            split_cont = cont.split(" ")

            if (
                split_cont[0] == "WORKDIR"
                or split_cont[0] == "ADD"
                or split_cont[0] == "COPY"
            ):

                r = re.compile(re_filter)
                con_filters = list(filter(r.match, split_cont))

                if len(con_filters) == 0:
                    new_cont.append(cont)
            else:
                new_cont.append(cont)

        return new_cont

    def check_dockerfile_ci_env(self, dock_cont, dock_fltr):
        """Check if any env variables config on dockerfile.

        Fix env variables component whatever exist or not.

        Args:
            dock_cont (str): content of dockerfile template.
            re_filter (str): string for regex filter.

        Returns:
            list: list of new content for dockerfile.

        """
        new_cont = []

        for cont in dock_cont:
            if cont == "":
                continue
            if cont != dock_fltr and cont != dock_fltr.strip("{}"):
                new_cont.append(cont)

        return new_cont

    def generate_dockerfile(self):
        """All processes of modifying dockerfile content."""
        try:
            for c_comp in self.params["COMPONENTS"]:
                local_docker = "{local_src}/{comp_path}/Dockerfile".format(
                    local_src=self.params["LOCAL_SRC"],
                    comp_path=c_comp["COMP_PATH"]
                )

                if not os.path.exists(local_docker):
                    dock_cont = ["FROM python:3.7"]
                else:
                    dock_cont = self.read_dockerfile(local_docker)

                dock_cont = self.add_dockerfile_env(dock_cont, c_comp)
                self.write_dockerfile(local_docker, dock_cont)

        except KeyError:
            print("this pipeline haven't any components...")

    def main(self):
        """Generate all component configuration."""
        self.generate_ci_conf(".gitlab-ci.yml")
        self.generate_cloudbuild("cloudbuild.yaml")
        self.generate_dockerfile()


@click.command()
def main():
    """Entrypoint of kubextract."""
    args = prompt(QUES)
    gen = GenerateFrame(params_path=args["params_path"])
    gen.main()


if __name__ == "__main__":
    main(prog_name="kubextract")



import setuptools
import os

os.system("export PATH=$PATH:~/.local/bin")

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kubextract",
    version="1.1.4",
    author="Bentar Dwika",
    author_email="bentar@warungpintar.co",
    description="cli framework generator for developing ML on kubeflow",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={
        "kubextract": ["params/*", "templates/*"]
    },
    install_requires=[
        'kubernetes==11.0.0',
        'ruamel.yaml==0.16.10',
        'click==7.1.2',
        'PyInquirer==1.0.3',
        'prompt_toolkit==1.0.14',
        'urllib3==1.25.9',
        'requests==2.23.0',
        'kfp==1.2.0'
    ],
    entry_points={
        'console_scripts': [
            'kubextract = kubextract.utils.generate:main',
            'pipeline = kubextract.utils.pipeline:main'
        ],
    },
    python_requires='>=3.6',
)



import ruamel.yaml as yaml
import os


class GenerateMeta:
    """Create kfserving configuration for deploy inference.

    Args:
        params_path (str): location of component params.

    Attributes:
        params (dict): List of component params.

    """

    def __init__(self, params_path=None):
        """Construct for storing class attributes.

        Args:
            params_path (str): location of component params.

        """
        self.params = self.load_params(params_path)

    def load_params(self, path):
        """Read params file for component params.

        Args:
            path (str): location of params file.

        Returns:
            dict: component params.

        """
        with open(path, "r") as stream:
            params = yaml.safe_load(stream)

        return params

    def construct_yaml_str(self):
        """Modifying kfserving content.

        Returns:
            str: Content of kfserving file.

        """
        content = """
            apiVersion: serving.kubeflow.org/v1alpha2
            kind: InferenceService
            metadata:
              namespace: kubeflow
              labels:
                controller-tools.k8s.io: "1.0"
              name: {pipe_name}
              nodeSelector:
                {node_key}: {node_val}
            spec:
              default:
                predictor:
                  custom:
                    container:
                      image: {image_registry}/{img_name}:{tag}

        """.format(
            pipe_name=self.params["PIPE_NAME"],
            node_key=self.params["NODE_KEY"],
            node_val=self.params["NODE_VAL"],
            image_registry=self.params["IMAGE_REGISTRY"],
            img_name=self.params["IMG_NAME"],
            tag=os.getenv("CI_COMMIT_REF_NAME"),
        )

        return content

    def main(self):
        """All process modifying kfserving config file."""
        with open(os.getenv("KFSERVING_CONFIG"), "w") as outfile:
            content = self.construct_yaml_str()
            yaml_cont = yaml.load(content)
            yaml.dump(yaml_cont, outfile, default_flow_style=False)


if __name__ == "__main__":
    gen_meta = GenerateMeta(params_path=os.getenv("PARAMS_PATH"))
    gen_meta.main()



from kubextract.utils.generate import GenerateFrame
import ruamel.yaml as yaml
import re
import os


class CheckKubeFiles:
    """Testing Scenario for kubextract.

    Args:
        params_path (str): location of component params.

    Attributes:
        params (dict): List of component params.
        trgt_base (str): base target path for generate framework.
        params_path

    """

    def __init__(self, params_path=None):
        """Construct for storing class attributes.

        Args:
            params_path (str): location of component params.

        """
        self.params = self.load_params(params_path)
        self.params_path = params_path
        self.trgt_base = self.params["LOCAL_SRC"]

    def load_params(self, path):
        """Read params file for component params.

        Args:
            path (str): location of params file.

        Returns:
            dict: component params.

        """
        with open(path, "r") as stream:
            params = yaml.safe_load(stream)

        return params

    def read_file(self, path):
        """Read content file.

        Args:
            path (str): Location of file.

        Returns:
            list: List of content file.

        """
        with open(path, "r") as stream:
            data = stream.read()
            split_data = data.split("\n")

        return split_data

    def check_params_on_file(
            self, filename=None,
            comp_path=None, contents=None):
        """Check every params value of each config file.

        Args:
            filename (str): name of config file.
            comp_path (str): location of component path.
            contents (list): List of content that will be checked.

        """
        if comp_path is not None:
            trgt_file = "{trgt_base}/{comp_path}/{filename}".format(
                trgt_base=self.trgt_base, comp_path=comp_path,
                filename=filename
            )
        else:
            trgt_file = "{trgt_base}/{filename}".format(
                trgt_base=self.trgt_base, filename=filename
            )

        trgt_data = self.read_file(trgt_file)

        for cont in contents:
            status = []

            for value in trgt_data:
                print((cont, value))
                fltrs = re.findall(cont, value)

                if len(fltrs) > 0:
                    status.append(True)

            assert len(status) > 0

    def main(self):
        """List all content will be checked and check each files."""
        gen = GenerateFrame(params_path=self.params_path)
        gen.main()

        for comp in self.params["COMPONENTS"]:
            dock_content = []
            cb_content = []
            ci_content = [
                "build_{stage_name}".format(
                    stage_name=comp["STAGE_NAME"]
                ),
                "{local_src}/{comp_path}/cloudbuild.yaml".format(
                    local_src=self.params["LOCAL_SRC"],
                    comp_path=comp["COMP_PATH"]
                ),
                "_LOCAL_SRC={local_src}".format(
                    local_src=self.params["LOCAL_SRC"]
                ),
                "{local_src}/{comp_path}/Dockerfile".format(
                    local_src=self.params["LOCAL_SRC"],
                    comp_path=comp["COMP_PATH"]
                ),
                "{img_registry}/{img_name}".format(
                    img_registry=self.params["IMAGE_REGISTRY"],
                    img_name=comp["IMG_NAME"]
                ),
                "kubepipe_{pipe_name}".format(
                    pipe_name=self.params["PIPE_NAME"]
                ),
                "PARAMS_PATH={params_path}".format(
                    params_path=self.params_path
                )
            ]

            self.check_params_on_file(
                filename=".gitlab-ci.yml", contents=ci_content
            )

            try:
                for env_var in comp["CI_ENV"]:
                    cb_content.append(
                        "{env_var}=${{_{env_var}}}".format(
                            env_var=env_var
                        )
                    )
                    cb_content.append(
                        "_{env_var}: {env_var}".format(
                            env_var=env_var
                        )
                    )

                    self.check_params_on_file(
                        filename="cloudbuild.yaml",
                        comp_path=comp["COMP_PATH"],
                        contents=cb_content,
                    )

                    dock_content.append(
                        "ARG {env_var}".format(
                            env_var=env_var
                        )
                    )
                    dock_content.append(
                        "ENV {env_var}=${{{env_var}}}".format(
                            env_var=env_var
                        )
                    )
            except Exception:
                print("There's no CI_ENV")

            dock_content.append(
                "WORKDIR {doc_workdir}".format(
                    doc_workdir=comp["DOCKER_WORKDIR"]
                )
            )
            dock_content.append(
                "ADD {comp_path} {doc_workdir}".format(
                    comp_path=comp["COMP_PATH"],
                    doc_workdir=comp["DOCKER_WORKDIR"]
                )
            )
            dock_content.append(
                "ADD params.yml {doc_workdir}/params.yml".format(
                    doc_workdir=comp["DOCKER_WORKDIR"]
                )
            )

            self.check_params_on_file(
                filename="Dockerfile",
                comp_path=comp["COMP_PATH"],
                contents=dock_content,
            )


class TestAllScenario:
    """All scenario pytest."""

    def test_mnist_comp(self):
        """Check mnist component generation."""
        dir_name = os.path.dirname(os.path.abspath(__file__))
        dir_name = dir_name.replace("utils-test", "params")
        mnist_path = os.path.join(dir_name, "mnist.yml")

        check_files = CheckKubeFiles(params_path=mnist_path)
        check_files.main()

    def test_markdown_comp(self):
        """Check markdown component generation."""
        dir_name = os.path.dirname(os.path.abspath(__file__))
        dir_name = dir_name.replace("utils-test", "params")
        markdown_path = os.path.join(dir_name, "markdown.yml")

        check_files = CheckKubeFiles(params_path=markdown_path)
        check_files.main()
